package service

import (
	"context"
	"log/slog"
)
import (
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
)

import (
	"auth-server/common"
	pb "auth-server/rpc_idl/dr_protocol_pb"
)

var ProtocolService *Protocol

func init() {
	ProtocolService = new(Protocol)
}

type Protocol struct {
	basicClient    pb.BasicClient
	protocolClient pb.ProtocolClient
}

func (s *Protocol) Setup() {
	conn, err := grpc.NewClient(common.ServerConfig.Protocol, grpc.WithTransportCredentials(insecure.NewCredentials()))
	if err != nil {
		slog.Error("NewClient failed:", err)
		panic(err)
	}
	slog.Info("protocol conn success", "conn", conn.GetState())

	s.basicClient = pb.NewBasicClient(conn)
	s.protocolClient = pb.NewProtocolClient(conn)
}

func (s *Protocol) GetSoftwareInfo() *pb.SoftwareInfoReply {
	slog.Info("[rpc]SoftwareInfo...")
	r, err := s.basicClient.SoftwareInfo(context.Background(), &pb.EmptyRequest{})
	if err != nil {
		slog.Error("[rpc]SoftwareInfo failed:", err)
		return &pb.SoftwareInfoReply{
			Module:  "",
			Desc:    "",
			Build:   "",
			Version: "",
		}
	}
	return r
}

func (s *Protocol) GetPatientType(ctx context.Context, isEnabled *bool) (*pb.PatientTypeReply, error) {
	slog.Info("[rpc]GetPatientType...")
	r, err := s.protocolClient.GetPatientType(ctx, &pb.PatientTypeRequest{
		IsEnabled: isEnabled,
	})
	if err != nil {
		slog.Error("[rpc]GetPatientType failed:", err)
	}
	slog.Info("[rpc]GetPatientType result:", "rows", len(r.PatientTypeList))
	return r, err
}

func (s *Protocol) GetBodyPart(ctx context.Context, patientType *string, category *string, isEnabled *bool) (*pb.BodyPartReply, error) {
	slog.Info("[rpc]GetBodyPart...")
	r, err := s.protocolClient.GetBodyPart(ctx, &pb.BodyPartRequest{
		PatientType: patientType,
		Category:    category,
		IsEnabled:   isEnabled,
	})
	if err != nil {
		slog.Error("[rpc]GetPatientType failed:", err)
	}
	return r, err
}
